"""
******************************************************************************
This file is part of the lemmagen library. It gives support for lemmatization.
Copyright (C) 2011 Jernej Virag <jernej@virag.si>

The lemmagen library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
******************************************************************************
"""

DICTIONARY_SLOVENE="lib/data/lem-me-sl.bin"
DICTIONARY_ENGLISH="lib/data/lem-m-en.bin"
DICTIONARY_SERBIAN="lib/data/lem-me-sr.bin"
DICTIONARY_ITALIAN="lib/data/lem-m-it.bin"
DICTIONARY_ROMANIAN="lib/data/lem-me-ro.bin"
DICTIONARY_HUNGARIAN="lib/data/lem-me-hu.bin"
DICTIONARY_FRENCH="lib/data/lem-me-fr.bin"
DICTIONARY_GERMAN="lib/data/lem-m-ge.bin"
DICTIONARY_SPANISH="lib/data/lem-m-sp.bin"
DICTIONARY_CZECH="lib/data/lem-me-cs.bin"
DICTIONARY_BULGARIAN="lib/data/lem-me-bg.bin"
DICTIONARY_ESTONIAN="lib/data/lem-me-et.bin"